%option noyywrap
%option nomain

        char* input_file_name = NULL;
        char* output_file_name = NULL;
        char* file_write_mode = NULL;
	#define MAX_BUFFER 255
	char buffer[MAX_BUFFER];
	int arg_count = 0;
	char* arg_count_postfix;
	int current_level = 0;
	int current_line = 1;
	#define MAX_LEVELS 200
	char* level_ending_tag[MAX_LEVELS];
	char* level_operator[MAX_LEVELS];
	int current_part          = 0;
	int current_chapter       = 0;
	int current_section       = 0;
	int current_subsection    = 0;
	int current_subsubsection = 0;
	int html_part_heading = 1;
	int html_chapter_heading = 2;
	int html_section_heading = 3;
	int html_subsection_heading = 4;
	int html_subsubsection_heading = 5;
	int inside_math = 0;
	void setmath(void);
	void sethtml(void);

INICIO_MATH		"$$"
FIN_MATH		"$$"
IDENTIFICADOR		[a-z]'*
CONJUNTO		[A-Z]
NUMERO			[0-9]+([.'][0-9]+)?
VECTOR			"_"[a-zA-Z]'*
FUNCION			[a-z]"("
INICIO_NIVEL		"{"|"("|"["
FIN_NIVEL		"}"|")"|"]"

INDEX			"ndice("
BGCOLOR_TAG          \ BGCOLOR[ \t]*=[ \t]*\"[#0-9a-fA-F]+\"
TEXT_TAG             \ TEXT[ \t]*=[ \t]*\"[#0-9a-fA-F]+\"
BGPROPERTIES_TAG     \ BGPROPERTIES[ \t]*=[ \t]*\"[a-zA-Z]+\"

%x MATH

%%

{INICIO_MATH}		BEGIN(MATH);
<MATH>{FIN_MATH}	{
			  sethtml();
			  BEGIN(INITIAL);
			}
<MATH>","		{
			  if (current_level == 0) fprintf(yyout, "<mo>, </mo>");
			  else
			    {
			      if (arg_count > 0)
			        {
			          --arg_count;
			          if (arg_count == 0)
			            {
				      fprintf(yyout, "%s", arg_count_postfix);
			            }
			        }
			      fprintf(yyout, "%s", level_operator[current_level]);
			    }
			}
<MATH>{FUNCION}		{
			  setmath();
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "  </mrow>\n </mfenced>\n</mrow>\n";
			  strncpy(buffer, yytext, 1);
			  buffer[1] = 0;// Dejamos slo el nombre la funcin.
			  fprintf(yyout, "<mrow>\n <mi>%s</mi><mfenced open='(' close=')' separator=','>\n  <mrow>", buffer);
			}
<MATH>{IDENTIFICADOR}	{setmath(); fprintf(yyout, "<mi>%s</mi>", yytext);}
<MATH>{NUMERO}		{setmath(); fprintf(yyout, "<mn>%s</mn>", yytext);}
<MATH>"No"		{setmath(); fprintf(yyout, "<mo> &Not</mo>");}
<MATH>""		{setmath(); fprintf(yyout, "<mo> &Not</mo>");}
<MATH>"O"		{setmath(); fprintf(yyout, "<mo> &Or; </mo>");}
<MATH>"Y"		{setmath(); fprintf(yyout, "<mo> &And; </mo>");}
<MATH>"->"		{setmath(); fprintf(yyout, "<mo></mo><mo> &DoubleRightArrow; </mo><mo></mo>");}
<MATH>"<-"		{setmath(); fprintf(yyout, "<mo> &DoubleLeftArrow; </mo>");}
<MATH>"<->"		{setmath(); fprintf(yyout, "<mo> &DoubleLeftRightArrow; </mo>");}
<MATH>"Ptodo"		{setmath(); fprintf(yyout, "<mo> &ForAll; </mo>");}
<MATH>"Existe"		{setmath(); fprintf(yyout, "<mo> &Exists; </mo>");}
<MATH>"Tq"		{setmath(); fprintf(yyout, "<mo> | </mo>");}
<MATH>"="		{setmath(); fprintf(yyout, "<mo> = </mo>");}
<MATH>"="		{setmath(); fprintf(yyout, "<mo> &NotEqual; </mo>");}
<MATH>{CONJUNTO}	{setmath(); fprintf(yyout, "<mi>%s</mi>", yytext);}
<MATH>"Pert"		{setmath(); fprintf(yyout, "<mo> &Element; </mo>");}
<MATH>"Npert"		{setmath(); fprintf(yyout, "<mo> &NotElement; </mo>");}
<MATH>"Cont"		{setmath(); fprintf(yyout, "<mo> &Superset; </mo>");}
<MATH>"Econt"		{setmath(); fprintf(yyout, "<mo> &SupersetEqual; </mo>");}
<MATH>"Subcon"		{setmath(); fprintf(yyout, "<mo> &Subset; </mo>");}
<MATH>"Esubcon"		{setmath(); fprintf(yyout, "<mo> &subsetEqual; </mo>");}
<MATH>"Union"		{setmath(); fprintf(yyout, "<mo> &Union; </mo>");}
<MATH>"Inter"		{setmath(); fprintf(yyout, "<mo> &Intersection; </mo>");}
<MATH>"Vaco"		{sethtml(); fprintf(yyout, " &Oslash; ");}
<MATH>"..."		{setmath(); fprintf(yyout, "<mn> &TripleDot; </mn>");}

<MATH>"Prod("		{
			  sethtml();
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "<mo> &times </mo>";
			  level_ending_tag[current_level] = "</mrow>";
			}

<MATH>"~"		{setmath(); fprintf(yyout, "<mo> &Congruent; </mo>");}
<MATH>">="		{setmath(); fprintf(yyout, "<mo> &GreaterEqual; </mo>");}
<MATH>"<="		{setmath(); fprintf(yyout, "<mo> &le; </mo>");}
<MATH>">"		{setmath(); fprintf(yyout, "<mo> &Greater; </mo>");}
<MATH>"<"		{
			  if (inside_math)
			    {
			      sethtml();
			      fprintf(yyout, " &Less; ");
			      setmath();
			    }
			  else fprintf(yyout, "<mo> &Less; </mo>");
			}
<MATH>"Inv("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mn>-1</mn></msup>";
			}
<MATH>""		{setmath(); fprintf(yyout, "<mo>  </mo>");}

<MATH>"Aleph0"		{sethtml(); fprintf(yyout, "&alefsym;");}
<MATH>"Intc("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='[' close=']' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Inta("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=')' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Intca("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='[' close=')' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Intac("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=']' separator=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"Inf"		{sethtml(); fprintf(yyout, "&infin");}

<MATH>"Conj("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mover><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow><mo>_</mo></mover>";
			}
<MATH>"Inti("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mo>&Integral;</mo><mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			}
<MATH>"Int("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow><msubsup><mo>&Integral;</mo>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			  arg_count = 2;
			  arg_count_postfix = "</msubsup>";
			}
<MATH>"Cont("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mi>C</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Integ("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mi>L</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Prop"		{setmath(); fprintf(yyout, "&Proportional;");}
<MATH>"Bin("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='(' close=')'><mfrac>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfrac></mfenced>";
			}
<MATH>"Exp("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mi>e</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Pot("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mrow>");
			  level_operator[current_level] = "</mrow><mrow>";
			  level_ending_tag[current_level] = "</mrow></msup>";
			}
<MATH>"Weier("		{
			  sethtml(); fprintf(yyout, "&weierp;");
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			}
<MATH>"Gama("		{
			  sethtml(); fprintf(yyout, "&Gamma;");
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>";
			}
<MATH>"Neutro"		{sethtml(); fprintf(yyout, "<b>e</b>");}
<MATH>"Csuma"		{setmath(); fprintf(yyout, "&CirclePlus;");}
<MATH>"Cprod"		{setmath(); fprintf(yyout, "&CircleTimes;");}
<MATH>{VECTOR}		{setmath(); fprintf(yyout, "<mover><mi>%s</mi><mo>&RightArrow;</mo></mover>", yytext + 1);}
<MATH>"Norma("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='|' close='|'>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"T("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "<mi>t</mi></msup>";
			}
<MATH>"Det("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mfenced open='|' close='|'>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}
<MATH>"E("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mi>E</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"S("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<msup><mi>S</mi>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</msup>";
			}
<MATH>"Dist("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mo>d</mo><mfenced open='(' close=')' separators=','>");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mfenced>";
			}


<MATH>"$("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow>\n");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</mrow>\n";
			}
<MATH>"{"		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>{</mo>\n");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "<mo>}</mo></mrow>\n";
			}
<MATH>"("		{
			  setmath();
			  ++current_level;
			  fprintf(yyout, "<mrow><mo>(</mo>\n");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "<mo>)</mo></mrow>\n";
			}
<MATH>"["		{
			  setmath(); 
			  ++current_level;
			 fprintf(yyout, "<mrow><mo>[</mo>\n");
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "<mo>]</mo></mrow>\n";
			}
<MATH>{FIN_NIVEL}	{
			  if (current_level == 0)
			    {
			      fprintf(stderr, "Unmatched parentheses at line: %d\n", current_line);
			    }
			  else
			    {
			      setmath(); fprintf(yyout, "%s", level_ending_tag[current_level]);
			      --current_level;
			    }
			}


"("			{
			  ++current_level;
			  fprintf(yyout, yytext);
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = ")";
			}
")"			{
			  if (current_level == 0)
			    {
			      fprintf(stderr, "Unmatched parentheses at line: %d\n", current_line);
			    }
			  else
			    {
			      fprintf(yyout, "%s", level_ending_tag[current_level]);
			      --current_level;
			    }
			}
"Parte("[^)]+")"	{
			  ++current_part;
			  current_chapter = 0;
			  current_section = 0;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Parte("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Parte_%d_%s'></a><center><h%d>Parte %d.<br>%s</h%d></center><hr size=4>", current_part, buffer, html_part_heading, current_part, buffer, html_part_heading);
			}
"Captulo("		{
			  ++current_chapter;
			  current_section = 0;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  sethtml();
			  fprintf(yyout, "<a name='Captulo_%d.%d'></a><center><h2>CAPTULO %d<br>", current_part, current_chapter, current_chapter);
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</h2></center><hr>";
			}
"Seccin("		{
			  ++current_section;
			  current_subsection = 0;
			  current_subsubsection = 0;
			  sethtml();
			  fprintf(yyout, "<a name='Seccin_%d.%d.%d'></a><center><h3>%d.%d ", current_part, current_chapter, current_section, current_chapter, current_section);
			  ++current_level;
			  level_operator[current_level] = "";
			  level_ending_tag[current_level] = "</h3></center>";
			}
"Subseccin("[^)]+")"	{
			  ++current_subsection;
			  current_subsubsection = 0;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Subseccin("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Subseccin_%d.%d.%d.%d_%s'></a><center><h%d>%d.%d.%d %s</h%d></center>", current_part, current_chapter, current_section, current_subsection, buffer, html_subsection_heading, current_chapter, current_section, current_subsection, buffer, html_subsection_heading);
			}
"Subsubseccin("[^)]+")" {
			  ++current_subsubsection;
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + strlen("Subsubseccin("));
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name='Subsubseccin_%d.%d.%d.%d.%d_%s'></a><center><h%d>%d.%d.%d.%d %s</h%d></center>", current_part, current_chapter, current_section, current_subsection, current_subsubsection, buffer, html_subsubsection_heading, current_chapter, current_section, current_subsection, current_subsubsection, buffer, html_subsubsection_heading);
			}
"ndice("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + 7);
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  fprintf(yyout, "<a name=\"%s\"><dfn>%s</dfn></a>", buffer, buffer);
			}
"Enlace("[^)]+")"	{
			  if (strlen(yytext) < MAX_BUFFER)
			    {
			      strcpy(buffer, yytext + 7);
			      buffer[strlen(buffer) - 1] = 0;
			    }
			  else
			    {
			      buffer[0] = 0;
			    }
			  if (buffer[0] != '#')
			    {
			      fprintf(yyout, "<a href=\"%s\">%s</a>", buffer, buffer);
			    }
			  else
			    {
			      fprintf(yyout, "<a href=\"%s\">%s</a>", buffer, buffer + 1);
			    }
			}
\n\n			{
			  current_line += 2;
			  if (inside_math)
			    {
			      sethtml(); fprintf(yyout, "\n<p>\n\n"); setmath();
			    }
			  else
			    {
			      fprintf(yyout, "\n<p>\n\n");
			    }
			}
\n			{
			  ++current_line;
			  //if (!inside_math) fprintf(yyout, "<br>");
			  fprintf(yyout, "%s", yytext);
			}





%%
void setmath(void)
  {
    if (!inside_math)
      {
        fprintf(yyout, "<math>");
	inside_math = 1;
      }
  }
void sethtml(void)
  {
    if (inside_math)
      {
        fprintf(yyout, "</math>");
	inside_math = 0;
      }
  }
yywrap() {return 1;};

void print_help(char *command_name);
int main(int argc, char *argv[]);

int main(int argc, char *argv[])
  {
    file_write_mode = "wb";
    if (argc > 1)
      {
        int index;
        for (index = 1; index <argc; ++index)
          {
            if (strcmp(argv[index], "-h") == 0
                || strcmp(argv[index], "--help") == 0)
              {
                print_help(argv[0]);
                return(0);
              }
            else if (strcmp(argv[index], "-a") == 0)
              {
                file_write_mode = "ab";
              }
            else if (strcmp(argv[index], "-o") == 0)
              {
                if (argc > index) {
                  output_file_name = argv[index + 1];
                  ++index;
                }
              }
            else if (strcmp(argv[index], "-f") == 0)
              {
                if (argc > index) {
                  input_file_name = argv[index + 1];
                  ++index;
                }
              }
            else if (*argv[index] == '-')
              {
                fprintf(stderr, "Unknown option. Use %s -h for help.\n", argv[0]);
                return 1;
              }
            else if (!input_file_name)
              {
                input_file_name = argv[index];
              }
            else if (!output_file_name)
              {
                output_file_name = argv[index];
              }
            else
              {
                fprintf(stderr, "Too many inputs. Use %s -h for help.\n", argv[0]);
                return 1;
              }
          }
	if (input_file_name && output_file_name)
          {
            if (strcmp(input_file_name, output_file_name) == 0)
              {
                fprintf(stderr, "Can not write on input file.\n");
                return 1;
              }
          }
        if (input_file_name)
          {
            fprintf(stderr, "Processing file: %s\n", input_file_name);
            if (!(yyin = fopen( input_file_name, "rb" )))
              {
                fprintf(stderr, "Cannot open file for reading: %s\n", input_file_name);
                if (output_file_name) fclose(yyout);
                return 1;
              }
          }
        if (output_file_name)
          {
            if (!(yyout = fopen( output_file_name, file_write_mode )))
              {
                fprintf(stderr, "Cannot open file for writing: %s\n", output_file_name);
                return 1;
              }
          }
      }
    if (yyout) fprintf(yyout,  "<html><head><title>QMathML</title></head>\n<body>\n\n");
    else       fprintf(stdout, "<html><head><title>QMathML</title></head>\n<body>\n\n");
    yylex();
    if (yyout) fprintf(yyout,  "\n\n</body>\n</html>");
    else       fprintf(stdout, "\n\n</body>\n</html>");
    fprintf(stderr, "Total source lines: %d\n", current_line);
    if (output_file_name) fclose(yyout);
    if (input_file_name) fclose(yyin);
  }

void print_help(char* command_name)
  {
    fprintf(stderr, "\n%s Copyright 1999 Alberto Gonzlez Palomo\n", command_name);
    fprintf(stderr, "Usage: %s [-a] [-f <input file>] [-o <output file>]\n", command_name);
    fprintf(stderr, "    -a   Adds the output to the end of <output file>\n");
    fprintf(stderr, "         The default is overwrite <output file>\n\n");
    fprintf(stderr, "Note: input and output can not be the same file.\n");
    fprintf(stderr, "      If no input nor output file name is given, the program\n");
    fprintf(stderr, "      uses stdin or stdout, respectively.\n");
  }













